package chembal;

import java.lang.*;
import java.util.*;
import java.util.regex.*;

/**
 * Holds one "atom" and its subscript
 *
 * <p>
 * Title: ChemBal
 * </p>
 *
 * <p>
 * Description: Balances chemical equations
 * </p>
 *
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 *
 * <p>
 * Company:
 * </p>
 *
 * @author Jeremy Drake
 * @version 1.0
 */
public class Element implements AtomContainer
{
	private ElementData eld;
	private String atomName = "";
	private int subscript = 0;

	/**
	 * Creates a new Element object.
	 */
	public Element()
	{
	}

	/**
	 * Creates a new Element object.
	 *
	 * @param atomName Name of the "atom".  Ex: "H", "O", "(NH3)"
	 * @param subscript Subscript of this element.  Ex: "H2" -> 2
	 */
	public Element(
		String atomName,
		int subscript)
	{
		this.atomName = atomName;
		this.subscript = subscript;
		eld = ElementDataConnector.getED(atomName);
	}

	/**
	 * Creates a new Element object.
	 *
	 * @param strElem The string representation of the container
	 */
	public Element(String strElem)
	{
		this.fromString(strElem);
	}

	/**
	 * Constructs the container based on the string representation
	 *
	 * @param strRep The string representation of the container
	 */
	public void fromString(String strRep)
	{
		Pattern p = Pattern.compile("([A-Z][a-z]*)(\\d*)");
		Matcher m = p.matcher(strRep);

		if (m.matches())
		{
			atomName = m.group(1);

			if ((m.group(2)).equals(""))
			{
				subscript = 1;
			}
			else
			{
				subscript = Integer.parseInt(m.group(2));
			}
		}

		eld = ElementDataConnector.getED(atomName);
	}

	/**
	 * Get the name of the atom of this element
	 *
	 * @return Name of the atom
	 */
	public String getAtomName()
	{
		return this.atomName;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public ElementData getElementData()
	{
		return eld;
	}

    public double getMolarMass()
    {
        return eld.getAtomWeight() * this.subscript;
    }

	/**
	 * Get the subscript of this element
	 *
	 * @return Subscript of the element
	 */
	public int getSubscript()
	{
		return this.subscript;
	}

	/**
	 * Get the number of atoms named <code>atomName</code> in this object. Since
	 * this is an <code>Element</code> object, this will return the subscript
	 * if this.atomName == atomName, 0 otherwise
	 *
	 * @param atomName Name of atom we're looking for
	 *
	 * @return Number of atoms of type atomName in this object
	 */
	public int getAtomCount(String atomName)
	{
		if (atomName.equals(this.atomName))
		{
			return this.subscript;
		}
		else
		{
			return 0;
		}
	}

	/**
	 * Returns true if this object contains an <code>atomName</code> atom,
	 * returns false otherwise
	 *
	 * @param atomName Name of atom we're looking for
	 *
	 * @return true if this object contains an <code>atomName</code> atom, false
	 * 				 otherwise
	 */
	public boolean containsAtom(String atomName)
	{
		return atomName.equals(this.atomName);
	}

	/**
	 * Main function for testing
	 *
	 * @param args command line arguments
	 */
	public static void main(String [] args)
	{
		Element element1 = new Element();
	}

	/**
	 * Returns a <code>Set</code> containing the string representations of all
	 * atoms contained in this object, which for an <code>Element</code> means
	 * a <code>Set</code> containing one string, <code>atomName</code>.
	 *
	 * @return The <code>Set</code> of all atoms in this object, in string form
	 */
	public Set getAtomSet()
	{
		Set st = new HashSet();
		st.add(atomName);

		return st;
	}

	/**
	 * Gets the string representation of this object
	 *
	 * @return The string representation of this object
	 */
	public String toString()
	{
		String retval = atomName;

		if (subscript > 1)
		{
			retval += Integer.toString(this.subscript);
		}

		return retval;
	}
}
