package chembal;

import java.lang.*;
import java.util.*;
import java.util.regex.*;

/**
 * Represents a chemical equation
 *
 * <p>
 * Title: ChemBal
 * </p>
 *
 * <p>
 * Description: Balances Chemical Equations
 * </p>
 *
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 *
 * <p>
 * Company:
 * </p>
 *
 * @author Jeremy Drake
 * @version 1.0
 *
 * @todo fix algorithm
 * @todo add proper support for ionic pseudo-atoms
 * @todo investigate possible bug in calculation, re: not being able to change
 * 			 more than one coeff for a given atom at a time
 */
public class Equation implements AtomContainer
{
	protected EquationSide leftSide;
	protected EquationSide rightSide;
	protected final int maxCoEff;

	/**
	 * Creates a new Equation object.
	 */
	public Equation()
	{
		rightSide = new EquationSide();
		leftSide = new EquationSide();
		maxCoEff = 10;
	}

	/**
	 * Creates a new Equation object.
	 *
	 * @param maxCoEff The maximum value to try as a coefficient
	 */
	public Equation(int maxCoEff)
	{
		this.maxCoEff = maxCoEff;
	}

	/**
	 * Creates a new Equation object.
	 *
	 * @param strEqn String representation of the <code>Equation</code> to
	 * 				construct
	 */
	public Equation(String strEqn)
	{
		this();
		this.fromString(strEqn);
	}

	/**
	 * Creates a new Equation object.
	 *
	 * @param strEqn String representation of the <code>Equation</code> to
	 * 				construct
	 * @param maxCoEff The maximum value to try as a coefficient
	 */
	public Equation(
		String strEqn,
		int maxCoEff)
	{
		this(maxCoEff);
		this.fromString(strEqn);
	}

	/**
	 * Generate an <code>Equation</code> from a string representation
	 *
	 * @param strEqn String representation of the <code>Equation</code> to
	 * 				generate
	 */
	public void fromString(String strEqn)
	{
		String [] sa = strEqn.split("->");

		leftSide.fromString(sa[0]);
		rightSide.fromString(sa[1]);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param args DOCUMENT ME!
	 */
	public static void main(String [] args)
	{
		Equation equation1 = new Equation();
	}

    public EquationSide getLeftSide()
    {
        return leftSide;
    }

    public EquationSide getRightSide()
    {
        return rightSide;
    }

	/**
	 * Get the number of atoms named <code>atomName</code> in this object.
	 *
	 * @param atomName Name of atom we're looking for
	 *
	 * @return Number of atoms of type atomName in this object
	 */
	public int getAtomCount(String atomName)
	{
		return rightSide.getAtomCount(atomName) + leftSide.getAtomCount(atomName);
	}

	/**
	 * Returns true if this object contains an <code>atomName</code> atom,
	 * returns false otherwise
	 *
	 * @param atomName Name of atom we're looking for
	 *
	 * @return true if this object contains an <code>atomName</code> atom, false
	 * 				 otherwise
	 */
	public boolean containsAtom(String atomName)
	{
		return rightSide.containsAtom(atomName) || leftSide.containsAtom(atomName);
	}

	/**
	 * Returns a <code>Set</code> containing the string representations of all
	 * atoms contained in this object
	 *
	 * @return The <code>Set</code> of all atoms in this object, in string form
	 */
	public Set getAtomSet()
	{
		Set st = new HashSet();
		st.addAll(leftSide.getAtomSet());
		st.addAll(rightSide.getAtomSet());

		return st;
	}

    public double getMolarMass()
    {
        double mm=leftSide.getMolarMass();
        mm+=rightSide.getMolarMass();
        return mm;
    }

	/**
	 * Actually balance the equation.  All of the heavy lifting occurs here.
	 *
	 * @return <code>true</code> if balancing succeded, <code>false</code>
	 * 				 otherwise
	 */

    /* TODO: B0rken */
	public boolean balance()
	{
		Set atst = leftSide.getAtomSet();
		Set atstR = rightSide.getAtomSet();

		// If there are atoms that occur on one side and not the other,
		// something's wrong.  Return failure status to caller
		// TODO: Make this work for the creation and destruction of
		// ionic "Elements" (CH4) usw.
		if (!atst.equals(atstR))
		{
			return false;
		}

		// Consolidate the sets.  Not strictly necessary ATM, because
		// of above 'if' statement, but good for future changes
		atst.addAll(atstR);

		// Clean up after ourselves.  Hopefully GC will take care of
		// this sooner if we kill our reference now
		atstR = null;

		Iterator atit = atst.iterator();

		// foreach atom in atom set
		while (atit.hasNext())
		{
			// Pull the string itself out of the iterator
			String atomToBalance = (String) (atit.next());

			// while not balanced
			while (
				leftSide.getAtomCount(atomToBalance) != rightSide.getAtomCount(
						atomToBalance))
			{
				//attempt to balance
				// get a list of molecules containing the atom
				// we're trying to balance from the left side
				List molsWithAtom = leftSide.moleculesContainingAtom(atomToBalance);

				// Add the molecules from the right side
				molsWithAtom.addAll(rightSide.moleculesContainingAtom(atomToBalance));
				Iterator molit = molsWithAtom.iterator();

				// foreach molecule in this list
				while (molit.hasNext())
				{
					// watch me pull a molecule out of my hat
					Molecule mol = (Molecule) (molit.next());

					// Loop thru, until either this molecule is
					// balanced, or we reach the max coeff
					while (
						mol.incCoefficient() < this.maxCoEff
							&& leftSide.getAtomCount(atomToBalance) != rightSide
							.getAtomCount(atomToBalance))
					{
						;
					}

					// if it didn't balance
					if (
						leftSide.getAtomCount(atomToBalance) != rightSide.getAtomCount(
								atomToBalance))
					{
						// roll back to 1
						mol.setCoefficient(1);

						// and continue to the next occurance
					}
					else
					{
						// else break out of this loop and
						// start over again to double check
						// the other mols
						break;
					}
				}

				//Start over again to make sure we didn't break
				//something we previously balanced
				atit = atst.iterator();
			}
		}

		// we did it!  tell the caller.
		return true;
	}

	/**
	 * Gets the string representation of this object
	 *
	 * @return The string representation of this object
	 */
	public String toString()
	{
		return leftSide.toString() + "->" + rightSide.toString();
	}

	/**
	 * Gets the string representation of the left side of this object
	 *
	 * @return The string representation of this object
	 */
	public String leftSideToString()
	{
		return leftSide.toString();
	}

	/**
	 * Gets the string representation of the right side of this object
	 *
	 * @return The string representation of this object
	 */
	public String rightSideToString()
	{
		return rightSide.toString();
	}
}
